import {get} from "lodash";

import createStyleSheet from "./create-stylesheet";

describe("createStyleSheet()",()=>{
  const sheet = createStyleSheet({test: {fontSize: "20px"}});
  it("should have no attached stylesheets",()=>{
    expect(document.styleSheets.length).toBe(0);
  });
  it("should add a stylesheet on attach",()=>{
    sheet.attach();
    expect(document.styleSheets.length).toBe(1);
  });
  it("should return a JSS stylesheet",()=>{
    expect(sheet).toHaveProperty("classes.test");
    expect(get(sheet,"classes.test")).not.toBe("test");
  });
  it("should remove the stylesheet on detach",()=>{
    sheet.detach();
    expect(document.styleSheets.length).toBe(0);
  });

});
describe("createStyleSheet(preserveClassNames)",()=>{

  const sheet = createStyleSheet({test: {fontSize: "20px"}},{preserveClassNames: true});
  const el = document.createElement("div");
  document.body.appendChild(el);

  it("should return the unmodified classname",()=>{
    expect(get(sheet,"classes.test")).toBe("test");
  });
  it("should produce the expected css",()=>{
    expect(sheet.toString().replace(/\s/g,"")).toBe(".test{font-size:20px;}");
  });

  it("should have no attached stylesheets",()=>{
    expect(document.styleSheets.length).toBe(0);
  });

  it("should add a stylesheet on attach",()=>{
    sheet.attach();
    expect(document.styleSheets.length).toBe(1);
  });
  it("should not have any effect on classless div",()=>{
    expect(window.getComputedStyle(el).getPropertyValue("font-size")).toBe("");
  });
  it("should have an effect on div with test class",()=>{
    el.setAttribute("class","test");
    expect(window.getComputedStyle(el).getPropertyValue("font-size")).toBe("20px");
  });
  it("should remove the stylesheet on detach",()=>{
    sheet.detach();
    expect(document.styleSheets.length).toBe(0);
  });
  it("should have remove the styling on the test div once detached",()=>{
    expect(window.getComputedStyle(el).getPropertyValue("font-size")).not.toBe("20px");
  });

});
