import {isString,isNumber,isArray,mergeWith} from "lodash";
import {rgba,darken as polishedDarken,lighten as polishedLighten, parseToRgb} from "polished";

export function parseUnitValue(v) {
  let value = 0;
  let unit = "";
  if (isNumber(v)) {
    value = v;
  }
  else if (isString(v)) {
    const match = v.match(/^(\d*(.\d+)?)(.*)/);
    if (match && match[1]) {
      value = parseFloat(match[1]);
      unit = match[3];
    }
  }
  return {value,unit};
}

export function parseUnitValues(...args) {
  let unit = "";
  const values = [];
  const parsedValues = args.map(parseUnitValue);
  const unitControl = parsedValues.find(v=>v.unit && v.unit !== "%") || parsedValues.find(v=>v.unit);
  if (unitControl) {
    unit = unitControl.unit;
  }

  parsedValues.forEach((v)=>{
    let value = v.value;
    if (v.unit === "%" && v !== unitControl) {
      value /= 100;
    }
    values.push(value);
  });

  return {unit,values};

}

export function multiply(...args) {
  const {unit,values} = parseUnitValues(...args);
  const [initial,...other] = values;
  let total = initial;
  other.forEach((v)=>{ total *= v; });

  return `${total}${unit}`;

}

export function add(...args) {
  const {unit,values} = parseUnitValues(...args);
  const [initial,...other] = values;
  let total = initial;
  other.forEach((v)=>{ total += v; });

  return `${total}${unit}`;

}

export function darken(color,amt) {
  return polishedDarken(amt,color);
}

export function lighten(color,amt) {
  return polishedLighten(amt,color);
}

export function getColorComponent(color,comp) {
  const c = parseToRgb(color);
  return c[comp];
}

export function red(color) {
  return getColorComponent(color,"red");
}

export function green(color) {
  return getColorComponent(color,"green");
}

export function blue(color) {
  return getColorComponent(color,"blue");
}

export function fuse(...objs) {
  const args = [{}].concat(objs).concat([(objVal,srcVal)=>{
    if (isArray(srcVal)) {
      return srcVal;
    }
    return undefined;
  }]);
  return mergeWith(...args);
}

export {rgba};
