
import {fuse,rgba} from "../lib/utils";
import {getProps} from "../index";
import {font,options,form,formFeedback,button} from "../variables";

import transition from "./mixins/transition";
import boxShadow from "./mixins/box-shadow";
import {formControlFocus,formValidationState} from "./mixins/forms";

//
// Textual form controls
//

function formControl(themeProps = {}) {

  const {formProps,fontProps,optionsProps,buttonProps} = getProps(themeProps,{form,font,options,button});

  return fuse(
    {
      display: "block",
      width: "100%",
      padding: `${buttonProps.inputPaddingY} ${buttonProps.inputPaddingX}`,
      fontSize: fontProps.sizeBase,
      lineHeight: buttonProps.inputLineHeight,
      color: formProps.inputColor,
      backgroundColor: formProps.inputBG,
      // Reset unusual Firefox-on-Android default style; see https://github.com/necolas/normalize.css/issues/214.
      backgroundImage: "none",
      backgroundClip: "padding-box",
      border: `${formProps.inputBtnBorderWidth} solid ${formProps.inputBorderColor}`,
      // Note: This has no effect on <select>s in some browsers, due to the limited stylability of `<select>`s in CSS.
      borderRadius: optionsProps.enableRounded ? formProps.inputBorderRadius : 0,
    },
    boxShadow(themeProps,formProps.inputBoxShadow),
    transition(themeProps,formProps.inputTransition),
    {
      // Unstyle the caret on `<select>`s in IE10+.
      "&::-ms-expand": {
        backgroundColor: "transparent",
        border: 0,
      },
    },

    // Customize the `:focus` state to imitate native WebKit styles.
    formControlFocus(themeProps),

    // Placeholder
    {
      "&::placeholder": {
        color: formProps.inputPlaceholderColor,
        // Override Firefox's unusual default opacity; see https://github.com/twbs/bootstrap/pull/11526.
        opacity: 1,
      },

      // Disabled and read-only inputs
      //
      // HTML5 says that controls under a fieldset > legend:first-child won't be
      // disabled if the fieldset is disabled. Due to implementation difficulty, we
      // don't honor that edge case; we style them as disabled anyway.
      "&:disabled, &[readonly]": {
        backgroundColor: formProps.inputDisabledBG,
        // iOS fix for unreadable disabled content; see https://github.com/twbs/bootstrap/issues/11655.
        opacity: 1,
      },


      "&:not(input):not(div):not(textarea)": { // hack to support select.form-control
        "&:not([size]):not([multiple])": {
          height: formProps.inputHeight,
        },

        "&:focus::-ms-value": {
          // Suppress the nested default white text on blue background highlight given to
          // the selected option text when the (still closed) <select> receives focus
          // in IE and (under certain conditions) Edge, as it looks bad and cannot be made to
          // match the appearance of the native widget.
          // See https://github.com/twbs/bootstrap/issues/19398.
          color: formProps.inputColor,
          backgroundColor: formProps.inputBG,
        },
      },
    }
  );
}

formControl.className = "form-control";

export function formControlFile() {
  return {
    display: "block",
  };
}
formControlFile.className = "form-control-file";

export function formControlRange() {
  return {
    display: "block",
  };
}
formControlRange.className = "form-control-range";


//
// Labels
//

// For use with horizontal and inline forms, when you need the label text to
// align with the form controls.
export function colFormLabel(themeProps = {}) {
  const {formProps,buttonProps} = getProps(themeProps,{form,button});

  return {
    paddingTop: `calc(${buttonProps.inputPaddingY} + ${formProps.inputBtnBorderWidth})`,
    paddingBottom: `calc(${buttonProps.inputPaddingY} + ${formProps.inputBtnBorderWidth})`,
    marginBottom: 0, // Override the `<label>` default
    lineHeight: buttonProps.inputLineHeight,
  };
}
colFormLabel.className = "col-form-label";

export function colFormLabelLarge(themeProps = {}) {
  const {formProps,fontProps,buttonProps} = getProps(themeProps,{form,font,button});

  return {
    paddingTop: `calc(${buttonProps.inputPaddingYLarge} + ${formProps.inputBtnBorderWidth})`,
    paddingBottom: `calc(${buttonProps.inputPaddingYLarge} + ${formProps.inputBtnBorderWidth})`,
    fontSize: fontProps.sizeLarge,
    lineHeight: buttonProps.inputLineHeightLarge,
  };
}
colFormLabelLarge.className = "col-form-label-lg";

export function colFormLabelSmall(themeProps = {}) {
  const {formProps,fontProps,buttonProps} = getProps(themeProps,{form,font,button});
  return {
    paddingTop: `calc(${buttonProps.inputPaddingYSmall} + ${formProps.inputBtnBorderWidth})`,
    paddingBottom: `calc(${buttonProps.inputPaddingYSmall} + ${formProps.inputBtnBorderWidth})`,
    fontSize: fontProps.sizeSmall,
    lineHeight: buttonProps.inputLineHeightSmall,
  };
}
colFormLabelSmall.className = "col-form-label-sm";


//
// Legends
//

// For use with horizontal and inline forms, when you need the legend text to
// be the same size as regular labels, and to align with the form controls.
export function colFormLegend(themeProps = {}) {
  const {fontProps,buttonProps} = getProps(themeProps,{font,button});

  return {
    paddingTop: buttonProps.inputPaddingY,
    paddingBottom: buttonProps.inputPaddingY,
    marginBottom: 0,
    fontSize: fontProps.sizeBase,
  };
}
colFormLegend.className = "col-form-legend";


// Readonly controls as plain text
//
// Apply class to a readonly input to make it appear like regular plain
// text (without any border, background color, focus indicator)

export function formControlPlaintext(themeProps = {}) {

  const {formProps,buttonProps} = getProps(themeProps,{form,button});

  return {
    paddingTop: buttonProps.inputPaddingY,
    paddingBottom: buttonProps.inputPaddingY,
    marginBottom: 0, // match inputs if this class comes on inputs with default margins
    lineHeight: buttonProps.inputLineHeight,
    backgroundColor: "transparent",
    border: "solid transparent",
    borderWidth: `${formProps.inputBtnBorderWidth} 0`,

    "&.form-control-sm, &.form-control-lg": {
      paddingRight: 0,
      paddingLeft: 0,
    },
  };
}

formControlPlaintext.className = "form-control-plaintext";


// Form control sizing
//
// Build on `.form-control` with modifier classes to decrease or increase the
// height and font-size of form controls.
//
// The `.form-group-* form-control` variations are sadly duplicated to avoid the
// issue documented in https://github.com/twbs/bootstrap/issues/15074.

export function formControlSmall(themeProps = {}) {
  const {formProps,fontProps,buttonProps} = getProps(themeProps,{form,font,button});

  return {
    padding: `${buttonProps.inputPaddingYSmall} ${buttonProps.inputPaddingXSmall}`,
    fontSize: fontProps.sizeSmall,
    lineHeight: buttonProps.inputLineHeightSmall,
    borderRadius: formProps.inputBorderRadiusSmall,
    "&:not(input):not(div):not(textarea)": { // hack to support select.form-control
      "&:not([size]):not([multiple])": {
        height: formProps.inputHeightSmall,
      }
    }
  };
}
formControlSmall.className = "form-control-sm";



export function formControlLarge(themeProps = {}) {
  const {formProps,fontProps,buttonProps} = getProps(themeProps,{form,font,button});
  return {
    padding: `${buttonProps.inputPaddingYLarge} ${buttonProps.inputPaddingXLarge}`,
    fontSize: fontProps.sizeLarge,
    lineHeight: buttonProps.inputLineHeightLarge,
    borderRadius: formProps.inputBorderRadiusLarge,
    "&:not(input):not(div):not(textarea)": { // hack to support select.form-control
      "&:not([size]):not([multiple])": {
        height: formProps.inputHeightLarge,
      },
    },
  };
}

formControlLarge.className = "form-control-lg";



// Form groups
//
// Designed to help with the organization and spacing of vertical forms. For
// horizontal forms, use the predefined grid classes.

export function formGroup(themeProps = {}) {
  const {formProps} = getProps(themeProps,{form});
  return {
    marginBottom: formProps.groupMarginBottom,
  };
}
formGroup.className = "form-group";


export function formText(themeProps = {}) {
  const {formProps} = getProps(themeProps,{form});
  return {
    display: "block",
    marginTop: formProps.textMarginTop,
  };
}
formText.className = "form-text";


// Form grid
//
// Special replacement for our grid system's `.row` for tighter form layouts.

export function formRow() {
  return {
    display: "flex",
    flexWrap: "wrap",
    marginRight: "-5px",
    marginLeft: "-5px",

    '& > .col, > [class*="col-"]': {
      paddingRight: "5px",
      paddingLeft: "5px",
    },
  };
}
formRow.className = "form-row";

// Checkboxes and radios
//
// Indent the labels to position radios/checkboxes as hanging controls.

export function formCheck(themeProps = {}) {

  const {formProps,fontProps} = getProps(themeProps,{form,font});

  return {
    position: "relative",
    display: "block",
    marginBottom: formProps.checkMarginBottom,

    "&.disabled": {
      "&.form-check-label": {
        color: fontProps.textMuted,
      }
    }
  };
}
formCheck.className = "form-check";


export function formCheckLabel(themeProps = {}) {

  const {formProps} = getProps(themeProps,{form});

  return {
    paddingLeft: formProps.checkInputGutter,
    marginBottom: 0, // Override default `<label>` bottom margin
  };
}
formCheckLabel.className = "form-check-label";


export function formCheckInput(themeProps = {}) {

  const {formProps} = getProps(themeProps,{form});

  return {
    position: "absolute",
    marginTop: formProps.checkInputMarginY,
    marginLeft: `-${formProps.checkInputGutter}`,
  };
}
formCheckInput.className = "form-check-input";

// Radios and checkboxes on same line
export function formCheckInline(themeProps = {}) {
  const {formProps} = getProps(themeProps,{form});
  return {
    display: "inline-block",
    marginRight: formProps.checkInlineMarginX,
    "& .form-check-label": {
      verticalAlign: "middle",
    }
  };
}
formCheckInline.className = "form-check-inline";


// Form validation
//
// Provide feedback to users when form field values are valid or invalid. Works
// primarily for client-side validation via scoped `:invalid` and `:valid`
// pseudo-classes but also includes `.is-invalid` and `.is-valid` classes for
// server side validation.

export function formValidation(themeProps = {}) {
  const {formFeedbackProps} = getProps(themeProps,{formFeedback});
  return [
    formValidationState(themeProps,"valid",formFeedbackProps.validColor),
    formValidationState(themeProps,"invalid",formFeedbackProps.invalidColor),

    // legacy support for BS3 errors
    {
      className: "has-error",
      "& .form-control, & .form-control:focus": {
        borderColor: formFeedbackProps.invalidColor,
      },
      "& .form-control:focus": {
        boxShadow: `0 0 0 .2rem ${rgba(formFeedbackProps.invalidColor,0.25)}`,

      },
    },
  ];
}

// Inline forms
//
// Make forms appear inline(-block) by adding the `.form-inline` class. Inline
// forms begin stacked on extra small (mobile) devices and then go inline when
// viewports reach <768px.
//
// Requires wrapping inputs and labels with `.form-group` for proper display of
// default HTML form controls and our custom form controls (e.g., input groups).

export function formInline(/*themeProps = {}*/) {

  // const {formProps} = getProps(themeProps,{form});

  return {

    display: "flex",
    flexFlow: "row wrap",
    alignItems: "center", // Prevent shorter elements from growing to same height as others (e.g., small buttons growing to normal sized button height)

    // Because we use flex, the initial sizing of checkboxes is collapsed and
    // doesn't occupy the full-width (which is what we want for xs grid tier),
    // so we force that here.
    "& .form-check": {
      width: "100%",
    },

    // Kick in the inline
    // @include media-breakpoint-up(sm) {
    //   label {
    //     display: flex;
    //     align-items: center;
    //     justify-content: center;
    //     margin-bottom: 0;
    //   }
    //
    //   // Inline-block all the things for "inline"
    //   .form-group {
    //     display: flex;
    //     flex: 0 0 auto;
    //     flex-flow: row wrap;
    //     align-items: center;
    //     margin-bottom: 0;
    //   }
    //
    //   // Allow folks to *not* use `.form-group`
    //   .form-control {
    //     display: inline-block;
    //     width: auto; // Prevent labels from stacking above inputs in `.form-group`
    //     vertical-align: middle;
    //   }
    //
    //   // Make static controls behave like regular ones
    //   .form-control-plaintext {
    //     display: inline-block;
    //   }
    //
    //   .input-group {
    //     width: auto;
    //   }
    //
    //   .form-control-label {
    //     margin-bottom: 0;
    //     vertical-align: middle;
    //   }
    //
    //   // Remove default margin on radios/checkboxes that were used for stacking, and
    //   // then undo the floating of radios and checkboxes to match.
    //   .form-check {
    //     display: flex;
    //     align-items: center;
    //     justify-content: center;
    //     width: auto;
    //     margin-top: 0;
    //     margin-bottom: 0;
    //   }
    //   .form-check-label {
    //     padding-left: 0;
    //   }
    //   .form-check-input {
    //     position: relative;
    //     margin-top: 0;
    //     margin-right: $form-check-input-margin-x;
    //     margin-left: 0;
    //   }
    //
    //   // Custom form controls
    //   .custom-control {
    //     display: flex;
    //     align-items: center;
    //     justify-content: center;
    //     padding-left: 0;
    //   }
    //   .custom-control-indicator {
    //     position: static;
    //     display: inline-block;
    //     margin-right: $form-check-input-margin-x; // Flexbox alignment means we lose our HTML space here, so we compensate.
    //     vertical-align: text-bottom;
    //   }
    //
    //   // Re-override the feedback icon.
    //   .has-feedback .form-control-feedback {
    //     top: 0;
    //   }
    // }
  };
}
formInline.className = "form-inline";


export const formEssentials = [
  formControl,
  formControlFile,
  formControlRange,
  formControlPlaintext,
  formGroup,
  formText,
  formCheck,
  formCheckLabel,
  formCheckInput,
  formCheckInline,
];

export const formGrid = [
  colFormLabel,
  colFormLegend,
  formRow,
];

export const formSizes = [
  formControlSmall,
  formControlLarge,
];

export const formGridSizes = [
  colFormLabelLarge,
  colFormLabelSmall,
];

export const all = [
  formEssentials,
  formGrid,
  formSizes,
  formGridSizes,
  formValidation,
  formInline,
];

export default all;
