import {getProps} from "../../index";
import {button,options} from "../../variables";
import {darken,rgba,fuse} from "../../lib/utils";
import {colorYIQ} from "../functions";
import boxShadow from "./box-shadow";

// Button variants
//
// Easily pump out default styles, as well as :hover, :focus, :active,
// and disabled options for all buttons

export function buttonVariant(themeProps,background, border, activeBackground, activeBorder) {

  if (!activeBackground) {
    activeBackground = darken(background,0.075);
  }
  if (!activeBorder) {
    activeBorder = darken(border,0.1);
  }

  const {buttonProps,optionsProps} = getProps(themeProps,{button,options});
  return fuse(
    colorYIQ(background),
    {
      backgroundColor: background,
      borderColor: border,
    },
    boxShadow(themeProps,buttonProps.boxShadow),
    {

      "&:hover": fuse(
        colorYIQ(activeBackground),
        {
          backgroundColor: activeBackground,
          borderColor: activeBorder,
        }
      ),
      "&:focus,&.focus": {
        boxShadow: optionsProps.enableShadows ?
          `${buttonProps.boxShadow}, 0 0 0 3px ${rgba(border,0.5)}` :
          `0 0 0 3px ${rgba(border, 0.5)}`,
      },
      // Disabled comes first so active can properly restyle
      "&.disabled,&:disabled": {
        backgroundColor: background,
        borderColor: border,
      },
      "&:active,&.active": fuse({
        backgroundColor: activeBackground,
        backgroundImage: "none", // Remove the gradient for the pressed/active state
        borderColor: activeBorder,
      },boxShadow(themeProps,buttonProps.activeBoxShadow)),
    },
  );
}

export function buttonOutlineVariant(themeProps,color, colorHover = "#fff") {
  return {
    color,
    backgroundColor: "transparent",
    backgroundImage: "none",
    borderColor: color,

    "&:hover": {
      color: colorHover,
      backgroundColor: color,
      borderColor: color,
    },

    "&:focus,&.focus": {
      boxShadow: `0 0 0 3px ${rgba(color,0.5)}`,
    },

    "&.disabled,&:disabled": {
      color,
      backgroundColor: "transparent",
    },

    "&:active,&.active": {
      color: colorHover,
      backgroundColor: color,
      borderColor: color,
    }
  };
}

// Button sizes
export function buttonSize(themeProps,paddingY, paddingX, fontSize, lineHeight, borderRadius) {
  return {
    padding: `${paddingY} ${paddingX}`,
    fontSize,
    lineHeight,
    borderRadius,
  };
}
