// Form control focus state
//
// Generate a customized focus state and for any input with the specified color,
// which defaults to the `$input-focus-border-color` variable.
//
// We highly encourage you to not customize the default value, but instead use
// this to tweak colors on an as-needed basis. This aesthetic change is based on
// WebKit's default styles, but applicable to a wider range of browsers. Its
// usability and accessibility should be taken into account with any change.
//
// Example usage: change the default blue border and shadow to white for better
// contrast against a dark gray background.

import {getProps} from "../../index";
import {form} from "../../variables";
import {fuse,rgba} from "../../lib/utils";
import boxShadow from "../mixins/box-shadow";

export function formControlFocus(themeProps) {
  const {formProps} = getProps(themeProps,{form});
  return {
    "&:focus": fuse(
      {
        color: formProps.inputFocusColor,
        backgroundColor: formProps.inputFocusBG,
        borderColor: formProps.inputFocusBorderColor,
        outline: "none",
      },
      boxShadow(themeProps,formProps.inputFocusBoxShadow),
    ),
  };
}


export function formValidationState(themeProps, state, color) {

  return [
    {
      className: `${state}-feedback`,
      display: "none",
      marginTop: ".25rem",
      fontSize: ".875rem",
      color: color,
    },

    {
      className: `${state}-tooltip`,
      position: "absolute",
      top: "100%",
      zIndex: 5,
      display: "none",
      width: "250px",
      padding: ".5rem",
      marginTop: ".1rem",
      fontSize: ".875rem",
      lineHeight: 1,
      color: "#fff",
      backgroundColor: rgba(color,0.8),
      borderRadius: ".2rem",
    },

    {
      className: "was-validated",


      "& .form-control, & .custom-select": {
        [`&:${state}, &.is-${state}`]: {
          borderColor: color
        },

        "&:focus": {
          boxShadow: `0 0 0 .2rem ${rgba(color,0.25)}`,
        },

        [`& ~ .${state}-feedback, & ~ .${state}-tooltip`]: {
          display: "block",
        }
      },



      // TODO: redo check markup lol crap
      "& .form-check-input": {
        [`&:${state}, &.is-${state}`]: {
          "& + .form-check-label": {
            color: color,
          },
        },
      },

      // custom radios and checks
      "& .custom-control-input": {
        [`&:${state}, &.is-${state}`]: {
          "& ~ .custom-control-indicator": {
            backgroundColor: rgba(color,0.25),
          },
          "& ~ .custom-control-description": {
            color: color,
          },
        }
      },

      // custom file
      "& .custom-file-input": {
        [`&:${state}, &.is-${state}`]: {
          "& ~ .custom-file-control": {
            borderColor: color,

            "&::before": { borderColor: "inherit" },
          },
          "&:focus": {
            boxShadow: `0 0 0 .2rem ${rgba(color,0.25)}`,
          },
        },
      },
    }
  ];
}
