import {get} from "lodash";
import {createBootstrapStyleSheet} from "../index";
import {navBase,navPills,navJustified} from "./nav";
import {validProps,validateCSS} from "../lib/test-utils";


describe("nav",()=>{

  const sheet = createBootstrapStyleSheet([navBase,navPills,navJustified],{},{preserveClassNames: true});
  const css = sheet.toString();

  it("should return a valid style object",()=>{
    expect(validProps(sheet.rules.raw)).toBe(true);
  });
  it("should return valid css",()=>{
    return validateCSS(css).then((valid)=>{
      expect(valid).toBe(true);
    });
  });
  it("should match css snapshot",()=>{
    expect(sheet.toString()).toMatchSnapshot();
  });

  it("should have a nav class",()=>{
    expect(get(sheet,"classes.nav")).toBe("nav");
  });
  it("should have a nav-item class",()=>{
    expect(get(sheet,"classes.nav-item")).toBe("nav-item");
  });
  it("should have a nav-link class",()=>{
    expect(get(sheet,"classes.nav-link")).toBe("nav-link");
  });
  it("should have a nav-pills class",()=>{
    expect(get(sheet,"classes.nav-pills")).toBe("nav-pills");
  });
  it("should have a nav-fill class",()=>{
    expect(get(sheet,"classes.nav-fill")).toBe("nav-fill");
  });
  it("should have a nav-justified class",()=>{
    expect(get(sheet,"classes.nav-justified")).toBe("nav-justified");
  });


});
