import {merge,each} from "lodash";

import getProps from "../../lib/get-props";
import {font,color} from "../../variables";
import textTruncate from "../mixins/text-truncate";
import textHideMixin from "../mixins/text-hide";
import textEmphasisVariant from "../mixins/text-emphasis";


//
// Text
//

// Alignment
export function textAlignment(themeProps) {
  return [
    {
      className: "text-justify",
      textAlign: "justify !important"
    },
    {
      className: "text-nowrap",
      whiteSpace: "nowrap !important"
    },
    merge(
      {
        className: "text-truncate",
      },
      textTruncate(themeProps),
    ),
  ];
}

// TODO: Responsive alignment

// @each $breakpoint in map-keys($grid-breakpoints) {
//   @include media-breakpoint-up($breakpoint) {
//     $infix: breakpoint-infix($breakpoint, $grid-breakpoints);
//
//     .text#{$infix}-left   { text-align: left !important; }
//     .text#{$infix}-right  { text-align: right !important; }
//     .text#{$infix}-center { text-align: center !important; }
//   }
// }

// Transformation

export function textTransformation() {
  return [
    {
      className: "text-lowercase",
      textTransform: "lowercase !important"
    },
    {
      className: "text-uppercase",
      textTransform: "uppercase !important"
    },
    {
      className: "text-capitalize",
      textTransform: "capitalize !important"
    },
  ];
}

// Weight and italics

export function textFontTypes(themeProps) {

  const {fontProps} = getProps(themeProps,{font});

  return [
    {
      className: "font-weight-light",
      fontWeight: `${fontProps.weightLight} !important`,
    },
    {
      className: "font-weight-normal",
      fontWeight: `${fontProps.weightNormal} !important`,
    },
    {
      className: "font-weight-bold",
      fontWeight: `${fontProps.weightBold} !important`,
    },
    {
      className: "font-italic",
      fontStyle: "italic !important"
    },
  ];
}

// Contextual colors

export function textColors(themeProps) {

  const {colorProps,fontProps} = getProps(themeProps,{color,font});
  const {themeColors} = colorProps;

  const styles = [
    {
      className: "text-white",
      color: "#fff !important",
    }
  ];

  each(themeColors,(value,key)=>{
    styles.push(merge(
      {
        className: `text-${key}`
      },
      textEmphasisVariant(themeProps,value),
    ));
  });

  styles.push({
    className: "text-muted",
    color: `${fontProps.textMuted} !important`
  });

  return styles;
}

// Misc

export function textHide(themeProps) {
  return merge(
    {
      className: "text-hide",
    },
    textHideMixin(themeProps)
  );
}

export const textUtils = [
  textAlignment,
  textTransformation,
  textFontTypes,
  textColors,
  textHide,
];
